@@ -17,6 +17,7 @@
  * License along with this library. If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include "config.h"
 #import "GdkQuartzView.h"
 #include "gdkquartzwindow.h"
 #include "gdkprivate-quartz.h"
@@ -37,25 +38,25 @@
 
 -(BOOL)acceptsFirstResponder
 {
-  GDK_NOTE (EVENTS, g_print ("acceptsFirstResponder\n"));
+  GDK_NOTE (EVENTS, g_message ("acceptsFirstResponder"));
   return YES;
 }
 
 -(BOOL)becomeFirstResponder
 {
-  GDK_NOTE (EVENTS, g_print ("becomeFirstResponder\n"));
+  GDK_NOTE (EVENTS, g_message ("becomeFirstResponder"));
   return YES;
 }
 
 -(BOOL)resignFirstResponder
 {
-  GDK_NOTE (EVENTS, g_print ("resignFirstResponder\n"));
+  GDK_NOTE (EVENTS, g_message ("resignFirstResponder"));
   return YES;
 }
 
 -(void) keyDown: (NSEvent *) theEvent
 {
-  GDK_NOTE (EVENTS, g_print ("keyDown\n"));
+  GDK_NOTE (EVENTS, g_message ("keyDown"));
   [self interpretKeyEvents: [NSArray arrayWithObject: theEvent]];
 }
 
@@ -65,13 +66,13 @@
 
 -(NSUInteger)characterIndexForPoint: (NSPoint)aPoint
 {
-  GDK_NOTE (EVENTS, g_print ("characterIndexForPoint\n"));
+  GDK_NOTE (EVENTS, g_message ("characterIndexForPoint"));
   return 0;
 }
 
 -(NSRect)firstRectForCharacterRange: (NSRange)aRange actualRange: (NSRangePointer)actualRange
 {
-  GDK_NOTE (EVENTS, g_print ("firstRectForCharacterRange\n"));
+  GDK_NOTE (EVENTS, g_message ("firstRectForCharacterRange"));
   gint ns_x, ns_y;
   GdkRectangle *rect;
 
@@ -91,37 +92,37 @@
 
 -(NSArray *)validAttributesForMarkedText
 {
-  GDK_NOTE (EVENTS, g_print ("validAttributesForMarkedText\n"));
+  GDK_NOTE (EVENTS, g_message ("validAttributesForMarkedText"));
   return [NSArray arrayWithObjects: NSUnderlineStyleAttributeName, nil];
 }
 
 -(NSAttributedString *)attributedSubstringForProposedRange: (NSRange)aRange actualRange: (NSRangePointer)actualRange
 {
-  GDK_NOTE (EVENTS, g_print ("attributedSubstringForProposedRange\n"));
+  GDK_NOTE (EVENTS, g_message ("attributedSubstringForProposedRange"));
   return nil;
 }
 
 -(BOOL)hasMarkedText
 {
-  GDK_NOTE (EVENTS, g_print ("hasMarkedText\n"));
+  GDK_NOTE (EVENTS, g_message ("hasMarkedText"));
   return markedRange.location != NSNotFound && markedRange.length != 0;
 }
 
 -(NSRange)markedRange
 {
-  GDK_NOTE (EVENTS, g_print ("markedRange\n"));
+  GDK_NOTE (EVENTS, g_message ("markedRange"));
   return markedRange;
 }
 
 -(NSRange)selectedRange
 {
-  GDK_NOTE (EVENTS, g_print ("selectedRange\n"));
+  GDK_NOTE (EVENTS, g_message ("selectedRange"));
   return selectedRange;
 }
 
 -(void)unmarkText
 {
-  GDK_NOTE (EVENTS, g_print ("unmarkText\n"));
+  GDK_NOTE (EVENTS, g_message ("unmarkText"));
   gchar *prev_str;
   markedRange = selectedRange = NSMakeRange (NSNotFound, 0);
 
@@ -130,7 +131,7 @@
 
 -(void)setMarkedText: (id)aString selectedRange: (NSRange)newSelection replacementRange: (NSRange)replacementRange
 {
-  GDK_NOTE (EVENTS, g_print ("setMarkedText\n"));
+  GDK_NOTE (EVENTS, g_message ("setMarkedText"));
   const char *str;
   gchar *prev_str;
 
@@ -158,9 +159,9 @@
   g_object_set_data (G_OBJECT (gdk_window), TIC_SELECTED_LEN,
 		     GUINT_TO_POINTER (selectedRange.length));
 
-  GDK_NOTE (EVENTS, g_print ("setMarkedText: set %s (%p, nsview %p): %s\n",
-			     TIC_MARKED_TEXT, gdk_window, self,
-			     str ? str : "(empty)"));
+  GDK_NOTE (EVENTS, g_message ("setMarkedText: set %s (%p, nsview %p): %s",
+			       TIC_MARKED_TEXT, gdk_window, self,
+			       str ? str : "(empty)"));
 
   /* handle text input changes by mouse events */
   if (!GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (gdk_window),
@@ -172,14 +173,14 @@
 
 -(void)doCommandBySelector: (SEL)aSelector
 {
-  GDK_NOTE (EVENTS, g_print ("doCommandBySelector\n"));
+  GDK_NOTE (EVENTS, g_message ("doCommandBySelector"));
   if ([self respondsToSelector: aSelector])
     [self performSelector: aSelector];
 }
 
 -(void)insertText: (id)aString replacementRange: (NSRange)replacementRange
 {
-  GDK_NOTE (EVENTS, g_print ("insertText\n"));
+  GDK_NOTE (EVENTS, g_message ("insertText"));
   const char *str;
   NSString *string;
   gchar *prev_str;
@@ -209,7 +210,7 @@
    }
 
   g_object_set_data_full (G_OBJECT (gdk_window), TIC_INSERT_TEXT, g_strdup (str), g_free);
-  GDK_NOTE (EVENTS, g_print ("insertText: set %s (%p, nsview %p): %s\n",
+  GDK_NOTE (EVENTS, g_message ("insertText: set %s (%p, nsview %p): %s",
 			     TIC_INSERT_TEXT, gdk_window, self,
 			     str ? str : "(empty)"));
 
@@ -226,314 +227,314 @@
 
 -(void)deleteBackward: (id)sender
 {
-  GDK_NOTE (EVENTS, g_print ("deleteBackward\n"));
+  GDK_NOTE (EVENTS, g_message ("deleteBackward"));
   g_object_set_data (G_OBJECT (gdk_window), GIC_FILTER_KEY,
 		     GUINT_TO_POINTER (GIC_FILTER_PASSTHRU));
 }
 
 -(void)deleteForward: (id)sender
 {
-  GDK_NOTE (EVENTS, g_print ("deleteForward\n"));
+  GDK_NOTE (EVENTS, g_message ("deleteForward"));
   g_object_set_data (G_OBJECT (gdk_window), GIC_FILTER_KEY,
 		     GUINT_TO_POINTER (GIC_FILTER_PASSTHRU));
 }
 
 -(void)deleteToBeginningOfLine: (id)sender
 {
-  GDK_NOTE (EVENTS, g_print ("deleteToBeginningOfLine\n"));
+  GDK_NOTE (EVENTS, g_message ("deleteToBeginningOfLine"));
   g_object_set_data (G_OBJECT (gdk_window), GIC_FILTER_KEY,
 		     GUINT_TO_POINTER (GIC_FILTER_PASSTHRU));
 }
 
 -(void)deleteToEndOfLine: (id)sender
 {
-  GDK_NOTE (EVENTS, g_print ("deleteToEndOfLine\n"));
+  GDK_NOTE (EVENTS, g_message ("deleteToEndOfLine"));
   g_object_set_data (G_OBJECT (gdk_window), GIC_FILTER_KEY,
 		     GUINT_TO_POINTER (GIC_FILTER_PASSTHRU));
 }
 
 -(void)deleteWordBackward: (id)sender
 {
-  GDK_NOTE (EVENTS, g_print ("deleteWordBackward\n"));
+  GDK_NOTE (EVENTS, g_message ("deleteWordBackward"));
   g_object_set_data (G_OBJECT (gdk_window), GIC_FILTER_KEY,
 		     GUINT_TO_POINTER (GIC_FILTER_PASSTHRU));
 }
 
 -(void)deleteWordForward: (id)sender
 {
-  GDK_NOTE (EVENTS, g_print ("deleteWordForward\n"));
+  GDK_NOTE (EVENTS, g_message ("deleteWordForward"));
   g_object_set_data (G_OBJECT (gdk_window), GIC_FILTER_KEY,
 		     GUINT_TO_POINTER (GIC_FILTER_PASSTHRU));
 }
 
 -(void)insertBacktab: (id)sender
 {
-  GDK_NOTE (EVENTS, g_print ("insertBacktab\n"));
+  GDK_NOTE (EVENTS, g_message ("insertBacktab"));
   g_object_set_data (G_OBJECT (gdk_window), GIC_FILTER_KEY,
 		     GUINT_TO_POINTER (GIC_FILTER_PASSTHRU));
 }
 
 -(void)insertNewline: (id)sender
 {
-  GDK_NOTE (EVENTS, g_print ("insertNewline\n"));
+  GDK_NOTE (EVENTS, g_message ("insertNewline"));
   g_object_set_data (G_OBJECT (gdk_window), GIC_FILTER_KEY, GUINT_TO_POINTER (GIC_FILTER_PASSTHRU));
 }
 
 -(void)insertTab: (id)sender
 {
-  GDK_NOTE (EVENTS, g_print ("insertTab\n"));
+  GDK_NOTE (EVENTS, g_message ("insertTab"));
   g_object_set_data (G_OBJECT (gdk_window), GIC_FILTER_KEY,
 		     GUINT_TO_POINTER (GIC_FILTER_PASSTHRU));
 }
 
 -(void)moveBackward: (id)sender
 {
-  GDK_NOTE (EVENTS, g_print ("moveBackward\n"));
+  GDK_NOTE (EVENTS, g_message ("moveBackward"));
   g_object_set_data (G_OBJECT (gdk_window), GIC_FILTER_KEY,
 		     GUINT_TO_POINTER (GIC_FILTER_PASSTHRU));
 }
 
 -(void)moveBackwardAndModifySelection: (id)sender
 {
-  GDK_NOTE (EVENTS, g_print ("moveBackwardAndModifySelection\n"));
+  GDK_NOTE (EVENTS, g_message ("moveBackwardAndModifySelection"));
   g_object_set_data (G_OBJECT (gdk_window), GIC_FILTER_KEY,
 		     GUINT_TO_POINTER (GIC_FILTER_PASSTHRU));
 }
 
 -(void)moveDown: (id)sender
 {
-  GDK_NOTE (EVENTS, g_print ("moveDown\n"));
+  GDK_NOTE (EVENTS, g_message ("moveDown"));
   g_object_set_data (G_OBJECT (gdk_window), GIC_FILTER_KEY,
 		     GUINT_TO_POINTER (GIC_FILTER_PASSTHRU));
 }
 
 -(void)moveDownAndModifySelection: (id)sender
 {
-  GDK_NOTE (EVENTS, g_print ("moveDownAndModifySelection\n"));
+  GDK_NOTE (EVENTS, g_message ("moveDownAndModifySelection"));
   g_object_set_data (G_OBJECT (gdk_window), GIC_FILTER_KEY,
 		     GUINT_TO_POINTER (GIC_FILTER_PASSTHRU));
 }
 
 -(void)moveForward: (id)sender
 {
-  GDK_NOTE (EVENTS, g_print ("moveForward\n"));
+  GDK_NOTE (EVENTS, g_message ("moveForward"));
   g_object_set_data (G_OBJECT (gdk_window), GIC_FILTER_KEY,
 		     GUINT_TO_POINTER (GIC_FILTER_PASSTHRU));
 }
 
 -(void)moveForwardAndModifySelection: (id)sender
 {
-  GDK_NOTE (EVENTS, g_print ("moveForwardAndModifySelection\n"));
+  GDK_NOTE (EVENTS, g_message ("moveForwardAndModifySelection"));
   g_object_set_data (G_OBJECT (gdk_window), GIC_FILTER_KEY,
 		     GUINT_TO_POINTER (GIC_FILTER_PASSTHRU));
 }
 
 -(void)moveLeft: (id)sender
 {
-  GDK_NOTE (EVENTS, g_print ("moveLeft\n"));
+  GDK_NOTE (EVENTS, g_message ("moveLeft"));
   g_object_set_data (G_OBJECT (gdk_window), GIC_FILTER_KEY,
 		     GUINT_TO_POINTER (GIC_FILTER_PASSTHRU));
 }
 
 -(void)moveLeftAndModifySelection: (id)sender
 {
-  GDK_NOTE (EVENTS, g_print ("moveLeftAndModifySelection\n"));
+  GDK_NOTE (EVENTS, g_message ("moveLeftAndModifySelection"));
   g_object_set_data (G_OBJECT (gdk_window), GIC_FILTER_KEY,
 		     GUINT_TO_POINTER (GIC_FILTER_PASSTHRU));
 }
 
 -(void)moveRight: (id)sender
 {
-  GDK_NOTE (EVENTS, g_print ("moveRight\n"));
+  GDK_NOTE (EVENTS, g_message ("moveRight"));
   g_object_set_data (G_OBJECT (gdk_window), GIC_FILTER_KEY,
 		     GUINT_TO_POINTER (GIC_FILTER_PASSTHRU));
 }
 
 -(void)moveRightAndModifySelection: (id)sender
 {
-  GDK_NOTE (EVENTS, g_print ("moveRightAndModifySelection\n"));
+  GDK_NOTE (EVENTS, g_message ("moveRightAndModifySelection"));
   g_object_set_data (G_OBJECT (gdk_window), GIC_FILTER_KEY,
 		     GUINT_TO_POINTER (GIC_FILTER_PASSTHRU));
 }
 
 -(void)moveToBeginningOfDocument: (id)sender
 {
-  GDK_NOTE (EVENTS, g_print ("moveToBeginningOfDocument\n"));
+  GDK_NOTE (EVENTS, g_message ("moveToBeginningOfDocument"));
   g_object_set_data (G_OBJECT (gdk_window), GIC_FILTER_KEY,
 		     GUINT_TO_POINTER (GIC_FILTER_PASSTHRU));
 }
 
 -(void)moveToBeginningOfDocumentAndModifySelection: (id)sender
 {
-  GDK_NOTE (EVENTS, g_print ("moveToBeginningOfDocumentAndModifySelection\n"));
+  GDK_NOTE (EVENTS, g_message ("moveToBeginningOfDocumentAndModifySelection"));
   g_object_set_data (G_OBJECT (gdk_window), GIC_FILTER_KEY,
 		     GUINT_TO_POINTER (GIC_FILTER_PASSTHRU));
 }
 
 -(void)moveToBeginningOfLine: (id)sender
 {
-  GDK_NOTE (EVENTS, g_print ("moveToBeginningOfLine\n"));
+  GDK_NOTE (EVENTS, g_message ("moveToBeginningOfLine"));
   g_object_set_data (G_OBJECT (gdk_window), GIC_FILTER_KEY,
 		     GUINT_TO_POINTER (GIC_FILTER_PASSTHRU));
 }
 
 -(void)moveToBeginningOfLineAndModifySelection: (id)sender
 {
-  GDK_NOTE (EVENTS, g_print ("moveToBeginningOfLineAndModifySelection\n"));
+  GDK_NOTE (EVENTS, g_message ("moveToBeginningOfLineAndModifySelection"));
   g_object_set_data (G_OBJECT (gdk_window), GIC_FILTER_KEY,
 		     GUINT_TO_POINTER (GIC_FILTER_PASSTHRU));
 }
 
 -(void)moveToEndOfDocument: (id)sender
 {
-  GDK_NOTE (EVENTS, g_print ("moveToEndOfDocument\n"));
+  GDK_NOTE (EVENTS, g_message ("moveToEndOfDocument"));
   g_object_set_data (G_OBJECT (gdk_window), GIC_FILTER_KEY,
 		     GUINT_TO_POINTER (GIC_FILTER_PASSTHRU));
 }
 
 -(void)moveToEndOfDocumentAndModifySelection: (id)sender
 {
-  GDK_NOTE (EVENTS, g_print ("moveToEndOfDocumentAndModifySelection\n"));
+  GDK_NOTE (EVENTS, g_message ("moveToEndOfDocumentAndModifySelection"));
   g_object_set_data (G_OBJECT (gdk_window), GIC_FILTER_KEY,
 		     GUINT_TO_POINTER (GIC_FILTER_PASSTHRU));
 }
 
 -(void)moveToEndOfLine: (id)sender
 {
-  GDK_NOTE (EVENTS, g_print ("moveToEndOfLine\n"));
+  GDK_NOTE (EVENTS, g_message ("moveToEndOfLine"));
   g_object_set_data (G_OBJECT (gdk_window), GIC_FILTER_KEY,
 		     GUINT_TO_POINTER (GIC_FILTER_PASSTHRU));
 }
 
 -(void)moveToEndOfLineAndModifySelection: (id)sender
 {
-  GDK_NOTE (EVENTS, g_print ("moveToEndOfLineAndModifySelection\n"));
+  GDK_NOTE (EVENTS, g_message ("moveToEndOfLineAndModifySelection"));
   g_object_set_data (G_OBJECT (gdk_window), GIC_FILTER_KEY,
 		     GUINT_TO_POINTER (GIC_FILTER_PASSTHRU));
 }
 
 -(void)moveUp: (id)sender
 {
-  GDK_NOTE (EVENTS, g_print ("moveUp\n"));
+  GDK_NOTE (EVENTS, g_message ("moveUp"));
   g_object_set_data (G_OBJECT (gdk_window), GIC_FILTER_KEY,
 		     GUINT_TO_POINTER (GIC_FILTER_PASSTHRU));
 }
 
 -(void)moveUpAndModifySelection: (id)sender
 {
-  GDK_NOTE (EVENTS, g_print ("moveUpAndModifySelection\n"));
+  GDK_NOTE (EVENTS, g_message ("moveUpAndModifySelection"));
   g_object_set_data (G_OBJECT (gdk_window), GIC_FILTER_KEY,
 		     GUINT_TO_POINTER (GIC_FILTER_PASSTHRU));
 }
 
 -(void)moveWordBackward: (id)sender
 {
-  GDK_NOTE (EVENTS, g_print ("moveWordBackward\n"));
+  GDK_NOTE (EVENTS, g_message ("moveWordBackward"));
   g_object_set_data (G_OBJECT (gdk_window), GIC_FILTER_KEY,
 		     GUINT_TO_POINTER (GIC_FILTER_PASSTHRU));
 }
 
 -(void)moveWordBackwardAndModifySelection: (id)sender
 {
-  GDK_NOTE (EVENTS, g_print ("moveWordBackwardAndModifySelection\n"));
+  GDK_NOTE (EVENTS, g_message ("moveWordBackwardAndModifySelection"));
   g_object_set_data (G_OBJECT (gdk_window), GIC_FILTER_KEY,
 		     GUINT_TO_POINTER (GIC_FILTER_PASSTHRU));
 }
 
 -(void)moveWordForward: (id)sender
 {
-  GDK_NOTE (EVENTS, g_print ("moveWordForward\n"));
+  GDK_NOTE (EVENTS, g_message ("moveWordForward"));
   g_object_set_data (G_OBJECT (gdk_window), GIC_FILTER_KEY,
 		     GUINT_TO_POINTER (GIC_FILTER_PASSTHRU));
 }
 
 -(void)moveWordForwardAndModifySelection: (id)sender
 {
-  GDK_NOTE (EVENTS, g_print ("moveWordForwardAndModifySelection\n"));
+  GDK_NOTE (EVENTS, g_message ("moveWordForwardAndModifySelection"));
   g_object_set_data (G_OBJECT (gdk_window), GIC_FILTER_KEY,
 		     GUINT_TO_POINTER (GIC_FILTER_PASSTHRU));
 }
 
 -(void)moveWordLeft: (id)sender
 {
-  GDK_NOTE (EVENTS, g_print ("moveWordLeft\n"));
+  GDK_NOTE (EVENTS, g_message ("moveWordLeft"));
   g_object_set_data (G_OBJECT (gdk_window), GIC_FILTER_KEY,
 		     GUINT_TO_POINTER (GIC_FILTER_PASSTHRU));
 }
 
 -(void)moveWordLeftAndModifySelection: (id)sender
 {
-  GDK_NOTE (EVENTS, g_print ("moveWordLeftAndModifySelection\n"));
+  GDK_NOTE (EVENTS, g_message ("moveWordLeftAndModifySelection"));
   g_object_set_data (G_OBJECT (gdk_window), GIC_FILTER_KEY,
 		     GUINT_TO_POINTER (GIC_FILTER_PASSTHRU));
 }
 
 -(void)moveWordRight: (id)sender
 {
-  GDK_NOTE (EVENTS, g_print ("moveWordRight\n"));
+  GDK_NOTE (EVENTS, g_message ("moveWordRight"));
   g_object_set_data (G_OBJECT (gdk_window), GIC_FILTER_KEY,
 		     GUINT_TO_POINTER (GIC_FILTER_PASSTHRU));
 }
 
 -(void)moveWordRightAndModifySelection: (id)sender
 {
-  GDK_NOTE (EVENTS, g_print ("moveWordRightAndModifySelection\n"));
+  GDK_NOTE (EVENTS, g_message ("moveWordRightAndModifySelection"));
   g_object_set_data (G_OBJECT (gdk_window), GIC_FILTER_KEY,
 		     GUINT_TO_POINTER (GIC_FILTER_PASSTHRU));
 }
 
 -(void)pageDown: (id)sender
 {
-  GDK_NOTE (EVENTS, g_print ("pageDown\n"));
+  GDK_NOTE (EVENTS, g_message ("pageDown"));
   g_object_set_data (G_OBJECT (gdk_window), GIC_FILTER_KEY,
 		     GUINT_TO_POINTER (GIC_FILTER_PASSTHRU));
 }
 
 -(void)pageDownAndModifySelection: (id)sender
 {
-  GDK_NOTE (EVENTS, g_print ("pageDownAndModifySelection\n"));
+  GDK_NOTE (EVENTS, g_message ("pageDownAndModifySelection"));
   g_object_set_data (G_OBJECT (gdk_window), GIC_FILTER_KEY,
 		     GUINT_TO_POINTER (GIC_FILTER_PASSTHRU));
 }
 
 -(void)pageUp: (id)sender
 {
-  GDK_NOTE (EVENTS, g_print ("pageUp\n"));
+  GDK_NOTE (EVENTS, g_message ("pageUp"));
   g_object_set_data (G_OBJECT (gdk_window), GIC_FILTER_KEY,
 		     GUINT_TO_POINTER (GIC_FILTER_PASSTHRU));
 }
 
 -(void)pageUpAndModifySelection: (id)sender
 {
-  GDK_NOTE (EVENTS, g_print ("pageUpAndModifySelection\n"));
+  GDK_NOTE (EVENTS, g_message ("pageUpAndModifySelection"));
   g_object_set_data (G_OBJECT (gdk_window), GIC_FILTER_KEY,
 		     GUINT_TO_POINTER (GIC_FILTER_PASSTHRU));
 }
 
 -(void)selectAll: (id)sender
 {
-  GDK_NOTE (EVENTS, g_print ("selectAll\n"));
+  GDK_NOTE (EVENTS, g_message ("selectAll"));
   g_object_set_data (G_OBJECT (gdk_window), GIC_FILTER_KEY,
 		     GUINT_TO_POINTER (GIC_FILTER_PASSTHRU));
 }
 
 -(void)selectLine: (id)sender
 {
-  GDK_NOTE (EVENTS, g_print ("selectLine\n"));
+  GDK_NOTE (EVENTS, g_message ("selectLine"));
   g_object_set_data (G_OBJECT (gdk_window), GIC_FILTER_KEY,
 		     GUINT_TO_POINTER (GIC_FILTER_PASSTHRU));
 }
 
 -(void)selectWord: (id)sender
 {
-  GDK_NOTE (EVENTS, g_print ("selectWord\n"));
+  GDK_NOTE (EVENTS, g_message ("selectWord"));
   g_object_set_data (G_OBJECT (gdk_window), GIC_FILTER_KEY,
 		     GUINT_TO_POINTER (GIC_FILTER_PASSTHRU));
 }
 
 -(void)noop: (id)sender
 {
-  GDK_NOTE (EVENTS, g_print ("noop\n"));
+  GDK_NOTE (EVENTS, g_message ("noop"));
 }
 
 /* --------------------------------------------------------------- */
@@ -662,7 +663,7 @@
   GdkWindowImplQuartz *impl = GDK_WINDOW_IMPL_QUARTZ (gdk_window->impl);
   NSRect rect;
 
-  if (!impl->toplevel)
+  if (!impl || !impl->toplevel)
     return;
 
   if (trackingRect)
