@@ -52,35 +52,6 @@ G_DEFINE_TYPE_WITH_CODE (GtkCellAccessible, gtk_cell_accessible, GTK_TYPE_ACCESS
                          G_IMPLEMENT_INTERFACE (ATK_TYPE_ACTION, atk_action_interface_init)
                          G_IMPLEMENT_INTERFACE (ATK_TYPE_COMPONENT, atk_component_interface_init))
 
-static void
-gtk_cell_accessible_object_finalize (GObject *obj)
-{
-  AtkRelationSet *relation_set;
-  AtkRelation *relation;
-  GPtrArray *target;
-  gpointer target_object;
-  gint i;
-
-  relation_set = atk_object_ref_relation_set (ATK_OBJECT (obj));
-  if (ATK_IS_RELATION_SET (relation_set))
-    {
-      relation = atk_relation_set_get_relation_by_type (relation_set,
-                                                        ATK_RELATION_NODE_CHILD_OF);
-      if (relation)
-        {
-          target = atk_relation_get_target (relation);
-          for (i = 0; i < target->len; i++)
-            {
-              target_object = g_ptr_array_index (target, i);
-              if (GTK_IS_CELL_ACCESSIBLE (target_object))
-                g_object_unref (target_object);
-            }
-        }
-      g_object_unref (relation_set);
-    }
-  G_OBJECT_CLASS (gtk_cell_accessible_parent_class)->finalize (obj);
-}
-
 static gint
 gtk_cell_accessible_get_index_in_parent (AtkObject *obj)
 {
@@ -99,6 +70,27 @@ gtk_cell_accessible_get_index_in_parent (AtkObject *obj)
   return gtk_cell_accessible_parent_get_child_index (GTK_CELL_ACCESSIBLE_PARENT (cell->priv->parent), cell);
 }
 
+static AtkRelationSet *
+gtk_cell_accessible_ref_relation_set (AtkObject *object)
+{
+  GtkCellAccessible *cell;
+  AtkRelationSet *relationset;
+  AtkObject *parent;
+
+  relationset = ATK_OBJECT_CLASS (gtk_cell_accessible_parent_class)->ref_relation_set (object);
+  if (relationset == NULL)
+    relationset = atk_relation_set_new ();
+
+  cell = GTK_CELL_ACCESSIBLE (object);
+  parent = gtk_widget_get_accessible (gtk_accessible_get_widget (GTK_ACCESSIBLE (cell)));
+
+  gtk_cell_accessible_parent_update_relationset (GTK_CELL_ACCESSIBLE_PARENT (parent),
+                                                 cell,
+                                                 relationset);
+
+  return relationset;
+}
+
 static AtkStateSet *
 gtk_cell_accessible_ref_state_set (AtkObject *accessible)
 {
@@ -156,12 +148,10 @@ static void
 gtk_cell_accessible_class_init (GtkCellAccessibleClass *klass)
 {
   AtkObjectClass *class = ATK_OBJECT_CLASS (klass);
-  GObjectClass *g_object_class = G_OBJECT_CLASS (klass);
-
-  g_object_class->finalize = gtk_cell_accessible_object_finalize;
 
   class->get_index_in_parent = gtk_cell_accessible_get_index_in_parent;
   class->ref_state_set = gtk_cell_accessible_ref_state_set;
+  class->ref_relation_set = gtk_cell_accessible_ref_relation_set;
   class->get_parent = gtk_cell_accessible_get_parent;
 }
 
@@ -429,19 +419,23 @@ _gtk_cell_accessible_state_changed (GtkCellAccessible    *cell,
 /*
  * gtk_cell_accessible_update_cache:
  * @cell: the cell that is changed
+ * @emit_signal: whether or not to notify the ATK bridge
  *
  * Notifies the cell that the values in the data in the row that
  * is used to feed the cell renderer with has changed. The
  * cell_changed function of @cell is called to send update
  * notifications for the properties it takes from its cell
- * renderer.
+ * renderer. If @emit_signal is TRUE, also notify the ATK bridge
+ * of the change. The bridge should be notified when an existing
+ * cell changes; not when a newly-created cell is being set up.
  *
  * Note that there is no higher granularity available about which
  * properties changed, so you will need to make do with this
  * function.
  **/
 void
-_gtk_cell_accessible_update_cache (GtkCellAccessible *cell)
+_gtk_cell_accessible_update_cache (GtkCellAccessible *cell,
+                                   gboolean           emit_signal)
 {
   GtkCellAccessibleClass *klass;
 
@@ -450,5 +444,5 @@ _gtk_cell_accessible_update_cache (GtkCellAccessible *cell)
   klass = GTK_CELL_ACCESSIBLE_GET_CLASS (cell);
 
   if (klass->update_cache)
-    klass->update_cache (cell);
+    klass->update_cache (cell, emit_signal);
 }
