@@ -204,7 +204,7 @@ gtk_application_load_resources (GtkApplication *application)
     gchar *iconspath;
 
     default_theme = gtk_icon_theme_get_default ();
-    iconspath = g_strconcat (base_path, "/icons", NULL);
+    iconspath = g_strconcat (base_path, "/icons/", NULL);
     gtk_icon_theme_add_resource_path (default_theme, iconspath);
     g_free (iconspath);
   }
@@ -1237,6 +1237,9 @@ gtk_application_uninhibit (GtkApplication *application,
  * Determines if any of the actions specified in @flags are
  * currently inhibited (possibly by another application).
  *
+ * Note that this information may not be available (for example
+ * when the application is running in a sandbox).
+ *
  * Returns: %TRUE if any of the actions specified in @flags are inhibited
  *
  * Since: 3.4
