@@ -133,6 +133,9 @@ static void dialog_destroy                          (GtkWidget         *widget,
 static void gtk_font_button_label_use_font          (GtkFontButton     *gfs);
 static void gtk_font_button_update_font_info        (GtkFontButton     *gfs);
 
+static void font_button_set_font_name               (GtkFontButton     *font_button,
+                                                     const char        *fontname);
+
 static guint font_button_signals[LAST_SIGNAL] = { 0 };
 
 static void
@@ -484,14 +487,16 @@ gtk_font_button_class_init (GtkFontButtonClass *klass)
    * The name of the currently selected font.
    *
    * Since: 2.4
+   *
+   * Deprecated: 3.22: Use the #GtkFontChooser::font property instead
    */
   g_object_class_install_property (gobject_class,
                                    PROP_FONT_NAME,
                                    g_param_spec_string ("font-name",
                                                         P_("Font name"),
                                                         P_("The name of the selected font"),
-                                                        P_("Sans 12"),
-                                                        GTK_PARAM_READWRITE));
+                                                        _("Sans 12"),
+                                                        GTK_PARAM_READWRITE | G_PARAM_DEPRECATED));
 
   /**
    * GtkFontButton:use-font:
@@ -568,7 +573,7 @@ gtk_font_button_class_init (GtkFontButtonClass *klass)
    *
    * Note that this signal is only emitted when the user
    * changes the font. If you need to react to programmatic font changes
-   * as well, use the notify::font-name signal.
+   * as well, use the notify::font signal.
    *
    * Since: 2.4
    */
@@ -662,7 +667,7 @@ gtk_font_button_set_property (GObject      *object,
       break;
     case GTK_FONT_CHOOSER_PROP_FONT:
     case PROP_FONT_NAME:
-      gtk_font_button_set_font_name (font_button, g_value_get_string (value));
+      font_button_set_font_name (font_button, g_value_get_string (value));
       break;
     case PROP_USE_FONT:
       gtk_font_button_set_use_font (font_button, g_value_get_boolean (value));
@@ -706,7 +711,7 @@ gtk_font_button_get_property (GObject    *object,
       break;
     case GTK_FONT_CHOOSER_PROP_FONT:
     case PROP_FONT_NAME:
-      g_value_set_string (value, gtk_font_button_get_font_name (font_button));
+      g_value_set_string (value, font_button->priv->fontname);
       break;
     case PROP_USE_FONT:
       g_value_set_boolean (value, gtk_font_button_get_use_font (font_button));
@@ -755,7 +760,7 @@ gtk_font_button_new (void)
 GtkWidget *
 gtk_font_button_new_with_font (const gchar *fontname)
 {
-  return g_object_new (GTK_TYPE_FONT_BUTTON, "font-name", fontname, NULL);
+  return g_object_new (GTK_TYPE_FONT_BUTTON, "font", fontname, NULL);
 } 
 
 /**
@@ -1003,6 +1008,7 @@ gtk_font_button_set_show_size (GtkFontButton *font_button,
  * Returns: an internal copy of the font name which must not be freed.
  *
  * Since: 2.4
+ * Deprecated: 3.22: Use gtk_font_chooser_get_font() instead
  */
 const gchar *
 gtk_font_button_get_font_name (GtkFontButton *font_button)
@@ -1012,6 +1018,16 @@ gtk_font_button_get_font_name (GtkFontButton *font_button)
   return font_button->priv->fontname;
 }
 
+static void
+font_button_set_font_name (GtkFontButton *font_button,
+                           const char    *fontname)
+{
+  PangoFontDescription *font_desc;
+
+  font_desc = pango_font_description_from_string (fontname);
+  gtk_font_button_take_font_desc (font_button, font_desc);
+}
+
 /**
  * gtk_font_button_set_font_name:
  * @font_button: a #GtkFontButton
@@ -1022,18 +1038,16 @@ gtk_font_button_get_font_name (GtkFontButton *font_button)
  * Returns: %TRUE
  *
  * Since: 2.4
+ * Deprecated: 3.22: Use gtk_font_chooser_set_font() instead
  */
 gboolean 
 gtk_font_button_set_font_name (GtkFontButton *font_button,
                                const gchar    *fontname)
 {
-  PangoFontDescription *font_desc;
-
   g_return_val_if_fail (GTK_IS_FONT_BUTTON (font_button), FALSE);
   g_return_val_if_fail (fontname != NULL, FALSE);
 
-  font_desc = pango_font_description_from_string (fontname);
-  gtk_font_button_take_font_desc (font_button, font_desc);
+  font_button_set_font_name (font_button, fontname);
 
   return TRUE;
 }
@@ -1164,6 +1178,127 @@ dialog_destroy (GtkWidget *widget,
   font_button->priv->font_dialog = NULL;
 } 
 
+static gchar *
+pango_font_description_to_css (PangoFontDescription *desc)
+{
+  GString *s;
+  PangoFontMask set;
+
+  s = g_string_new ("* { ");
+
+  set = pango_font_description_get_set_fields (desc);
+  if (set & PANGO_FONT_MASK_FAMILY)
+    {
+      g_string_append (s, "font-family: ");
+      g_string_append (s, pango_font_description_get_family (desc));
+      g_string_append (s, "; ");
+    }
+  if (set & PANGO_FONT_MASK_STYLE)
+    {
+      switch (pango_font_description_get_style (desc))
+        {
+        case PANGO_STYLE_NORMAL:
+          g_string_append (s, "font-style: normal; ");
+          break;
+        case PANGO_STYLE_OBLIQUE:
+          g_string_append (s, "font-style: oblique; ");
+          break;
+        case PANGO_STYLE_ITALIC:
+          g_string_append (s, "font-style: italic; ");
+          break;
+        }
+    }
+  if (set & PANGO_FONT_MASK_VARIANT)
+    {
+      switch (pango_font_description_get_variant (desc))
+        {
+        case PANGO_VARIANT_NORMAL:
+          g_string_append (s, "font-variant: normal; ");
+          break;
+        case PANGO_VARIANT_SMALL_CAPS:
+          g_string_append (s, "font-variant: small-caps; ");
+          break;
+        }
+    }
+  if (set & PANGO_FONT_MASK_WEIGHT)
+    {
+      switch (pango_font_description_get_weight (desc))
+        {
+        case PANGO_WEIGHT_THIN:
+          g_string_append (s, "font-weight: 100; ");
+          break;
+        case PANGO_WEIGHT_ULTRALIGHT:
+          g_string_append (s, "font-weight: 200; ");
+          break;
+        case PANGO_WEIGHT_LIGHT:
+        case PANGO_WEIGHT_SEMILIGHT:
+          g_string_append (s, "font-weight: 300; ");
+          break;
+        case PANGO_WEIGHT_BOOK:
+        case PANGO_WEIGHT_NORMAL:
+          g_string_append (s, "font-weight: 400; ");
+          break;
+        case PANGO_WEIGHT_MEDIUM:
+          g_string_append (s, "font-weight: 500; ");
+          break;
+        case PANGO_WEIGHT_SEMIBOLD:
+          g_string_append (s, "font-weight: 600; ");
+          break;
+        case PANGO_WEIGHT_BOLD:
+          g_string_append (s, "font-weight: 700; ");
+          break;
+        case PANGO_WEIGHT_ULTRABOLD:
+          g_string_append (s, "font-weight: 800; ");
+          break;
+        case PANGO_WEIGHT_HEAVY:
+        case PANGO_WEIGHT_ULTRAHEAVY:
+          g_string_append (s, "font-weight: 900; ");
+          break;
+        }
+    }
+  if (set & PANGO_FONT_MASK_STRETCH)
+    {
+      switch (pango_font_description_get_stretch (desc))
+        {
+        case PANGO_STRETCH_ULTRA_CONDENSED:
+          g_string_append (s, "font-stretch: ultra-condensed; ");
+          break;
+        case PANGO_STRETCH_EXTRA_CONDENSED:
+          g_string_append (s, "font-stretch: extra-condensed; ");
+          break;
+        case PANGO_STRETCH_CONDENSED:
+          g_string_append (s, "font-stretch: condensed; ");
+          break;
+        case PANGO_STRETCH_SEMI_CONDENSED:
+          g_string_append (s, "font-stretch: semi-condensed; ");
+          break;
+        case PANGO_STRETCH_NORMAL:
+          g_string_append (s, "font-stretch: normal; ");
+          break;
+        case PANGO_STRETCH_SEMI_EXPANDED:
+          g_string_append (s, "font-stretch: semi-expanded; ");
+          break;
+        case PANGO_STRETCH_EXPANDED:
+          g_string_append (s, "font-stretch: expanded; ");
+          break;
+        case PANGO_STRETCH_EXTRA_EXPANDED:
+          g_string_append (s, "font-stretch: extra-expanded; ");
+          break;
+        case PANGO_STRETCH_ULTRA_EXPANDED:
+          g_string_append (s, "font-stretch: ultra-expanded; ");
+          break;
+        }
+    }
+  if (set & PANGO_FONT_MASK_SIZE)
+    {
+      g_string_append_printf (s, "font-size: %dpt", pango_font_description_get_size (desc) / PANGO_SCALE);
+    }
+
+  g_string_append (s, "}");
+
+  return g_string_free (s, FALSE);
+}
+
 static void
 gtk_font_button_label_use_font (GtkFontButton *font_button)
 {
@@ -1183,7 +1318,7 @@ gtk_font_button_label_use_font (GtkFontButton *font_button)
   else
     {
       PangoFontDescription *desc;
-      gchar *font, *data;
+      gchar *data;
 
       if (!priv->provider)
         {
@@ -1198,14 +1333,10 @@ gtk_font_button_label_use_font (GtkFontButton *font_button)
       if (!priv->use_size)
         pango_font_description_unset_fields (desc, PANGO_FONT_MASK_SIZE);
 
-      font = pango_font_description_to_string (desc);
-      data = g_strconcat ("* { font: ",  font, "; }", NULL);
-      
+      data = pango_font_description_to_css (desc);
       gtk_css_provider_load_from_data (priv->provider, data, -1, NULL);
 
       g_free (data);
-      g_free (font);
-
       pango_font_description_free (desc);
     }
 }
