@@ -75,6 +75,9 @@
  * associated information.
  */
 
+/* We are using deprecated API, here, and we know that */
+#define GDK_DISABLE_DEPRECATION_WARNINGS
+
 #include "config.h"
 
 #include "gtkselection.h"
@@ -885,6 +888,10 @@ gtk_selection_clear_targets (GtkWidget *widget,
   if (GDK_IS_WAYLAND_DISPLAY (gtk_widget_get_display (widget)))
     gdk_wayland_selection_clear_targets (gtk_widget_get_display (widget), selection);
 #endif
+#ifdef GDK_WINDOWING_WIN32
+  if (GDK_IS_WIN32_DISPLAY (gtk_widget_get_display (widget)))
+    gdk_win32_selection_clear_targets (gtk_widget_get_display (widget), selection);
+#endif
 
   lists = g_object_get_data (G_OBJECT (widget), gtk_selection_handler_key);
   
@@ -1120,6 +1127,15 @@ gtk_selection_convert (GtkWidget *widget,
   display = gtk_widget_get_display (widget);
   owner_window = gdk_selection_owner_get_for_display (display, selection);
   
+#ifdef GDK_WINDOWING_WIN32
+  /* Special handling for DELETE requests,
+   * make sure this goes down into GDK layer.
+   */
+  if (GDK_IS_WIN32_DISPLAY (display) &&
+      target == gdk_atom_intern_static_string ("DELETE"))
+    owner_window = NULL;
+#endif
+
   if (owner_window != NULL)
     {
       GtkWidget *owner_widget;
