@@ -1915,13 +1915,13 @@ gtk_spin_button_default_output (GtkSpinButton *spin_button)
 /**
  * gtk_spin_button_configure:
  * @spin_button: a #GtkSpinButton
- * @adjustment: (allow-none):  a #GtkAdjustment
+ * @adjustment: (nullable): a #GtkAdjustment to replace the spin button’s
+ *     existing adjustment, or %NULL to leave its current adjustment unchanged
  * @climb_rate: the new climb rate
  * @digits: the number of decimal places to display in the spin button
  *
  * Changes the properties of an existing spin button. The adjustment,
- * climb rate, and number of decimal places are all changed accordingly,
- * after this function call.
+ * climb rate, and number of decimal places are updated accordingly.
  */
 void
 gtk_spin_button_configure (GtkSpinButton *spin_button,
@@ -2431,9 +2431,10 @@ gtk_spin_button_set_wrap (GtkSpinButton  *spin_button,
 
   if (priv->wrap != wrap)
     {
-       priv->wrap = wrap;
+      priv->wrap = wrap;
+      g_object_notify (G_OBJECT (spin_button), "wrap");
 
-       g_object_notify (G_OBJECT (spin_button), "wrap");
+      update_node_state (spin_button);
     }
 }
 
