@@ -211,8 +211,8 @@ gtk_switch_multipress_gesture_pressed (GtkGestureMultiPress *gesture,
   /* If the press didn't happen in the draggable handle,
    * cancel the pan gesture right away
    */
-  if ((priv->is_active && x <= allocation.width / 2) ||
-      (!priv->is_active && x > allocation.width / 2))
+  if ((priv->is_active && x <= allocation.width / 2.0) ||
+      (!priv->is_active && x > allocation.width / 2.0))
     gtk_gesture_set_state (priv->pan_gesture, GTK_EVENT_SEQUENCE_DENIED);
 }
 
@@ -982,7 +982,7 @@ gtk_switch_class_init (GtkSwitchClass *klass)
                   G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
                   G_STRUCT_OFFSET (GtkSwitchClass, activate),
                   NULL, NULL,
-                  _gtk_marshal_VOID__VOID,
+                  NULL,
                   G_TYPE_NONE, 0);
   widget_class->activate_signal = signals[ACTIVATE];
 
