@@ -35,6 +35,7 @@ struct _GtkIMContextWaylandGlobal
 {
   struct wl_display *display;
   struct wl_registry *registry;
+  uint32_t text_input_manager_wl_id;
   struct gtk_text_input_manager *text_input_manager;
   struct gtk_text_input *text_input;
   uint32_t enter_serial;
@@ -100,6 +101,9 @@ static const GtkIMContextInfo *info_list[] =
 static void
 reset_preedit (GtkIMContextWayland *context)
 {
+  if (context->preedit.text == NULL)
+    return;
+
   g_clear_pointer (&context->preedit.text, g_free);
   context->preedit.cursor_idx = 0;
   g_signal_emit_by_name (context, "preedit-changed");
@@ -205,8 +209,9 @@ registry_handle_global (void               *data,
 
   if (strcmp (interface, "gtk_text_input_manager") == 0)
     {
+      global->text_input_manager_wl_id = id;
       global->text_input_manager =
-        wl_registry_bind (global->registry, id,
+        wl_registry_bind (global->registry, global->text_input_manager_wl_id,
                           &gtk_text_input_manager_interface, 1);
       global->text_input =
         gtk_text_input_manager_get_text_input (global->text_input_manager,
@@ -223,11 +228,11 @@ registry_handle_global_remove (void               *data,
 {
   GtkIMContextWaylandGlobal *global = data;
 
-  gtk_text_input_destroy (global->text_input);
-  global->text_input = NULL;
+  if (id != global->text_input_manager_wl_id)
+    return;
 
-  gtk_text_input_manager_destroy (global->text_input_manager);
-  global->text_input_manager = NULL;
+  g_clear_pointer(&global->text_input, gtk_text_input_destroy);
+  g_clear_pointer(&global->text_input_manager, gtk_text_input_manager_destroy);
 }
 
 static const struct wl_registry_listener registry_listener = {
@@ -420,6 +425,9 @@ released_cb (GtkGestureMultiPress *gesture,
 {
   GtkInputHints hints;
 
+  if (!global->current)
+    return;
+
   g_object_get (context, "input-hints", &hints, NULL);
 
   if (n_press == 1 &&
