@@ -38,6 +38,17 @@ query_tooltip_cb (GtkWidget  *widget,
   return TRUE;
 }
 
+static gboolean
+draw_tooltip (GtkWidget *widget,
+              cairo_t   *cr,
+              gpointer   unused)
+{
+  cairo_set_source_rgb (cr, 0, 0, 1);
+  cairo_paint (cr);
+
+  return FALSE;
+}
+
 static gboolean
 query_tooltip_custom_cb (GtkWidget  *widget,
 			 gint        x,
@@ -46,10 +57,10 @@ query_tooltip_custom_cb (GtkWidget  *widget,
 			 GtkTooltip *tooltip,
 			 gpointer    data)
 {
-  GdkRGBA color = { 0, 0, 1, 1 };
   GtkWindow *window = gtk_widget_get_tooltip_window (widget);
 
-  gtk_widget_override_background_color (GTK_WIDGET (window), 0, &color);
+  gtk_widget_set_app_paintable (GTK_WIDGET (window), TRUE);
+  g_signal_connect (window, "draw", G_CALLBACK (draw_tooltip), NULL);
 
   return TRUE;
 }
@@ -330,6 +341,11 @@ main (int argc, char *argv[])
   gtk_widget_show (tooltip_button);
 
   gtk_widget_set_tooltip_window (button, GTK_WINDOW (tooltip_window));
+  gtk_window_set_type_hint (GTK_WINDOW (tooltip_window),
+                            GDK_WINDOW_TYPE_HINT_TOOLTIP);
+  gtk_window_set_transient_for (GTK_WINDOW (tooltip_window),
+                                GTK_WINDOW (window));
+
   g_signal_connect (button, "query-tooltip",
 		    G_CALLBACK (query_tooltip_custom_cb), NULL);
   g_object_set (button, "has-tooltip", TRUE, NULL);
